.g-group-list-entry
  .g-group-title
    font-size 17px

  .g-group-description
    color #666
    font-size 14px
    font-weight bold

  .g-space-used
    margin-top 2px

  .g-group-right-column
    border-left 1px solid #ddd
    padding-left 8px
    display inline
    width 290px
    font-size 14px
    float right
    color #909090

  .g-group-created-date
    color #444

  .g-group-privacy
    margin-top 3px

.g-groups-list-header
  padding-bottom 10px

  .g-group-pagination
    display inline-block
    >ul
      margin 0 15px 0 0

    .pagination>li>a
      float none
      line-height 1.5
      padding 7px 12px

  .g-group-search-form
    display inline-block
    margin-right 15px
